<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Storelocator\Controller\PickUpInStore;

use \Magento\Framework\Exception\NotFoundException;

/**
 * Responsible for loading page content.
 *
 * This is a basic controller that only loads the corresponding layout file. It may duplicate other such
 * controllers, and thus it is considered tech debt. This code duplication will be resolved in future releases.
 */
class ConfirmReceivedAction extends \Magento\Framework\App\Action\Action
{
    protected $jsonResultFactory;
    protected $helper;
    protected $_order;
    protected $_transportBuilder;
    protected $_storeManager;
    protected $_inlineTranslation;
    protected $_paymentHelper;
    protected $_addressRenderer;


    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \FiloBlu\Storelocator\Helper\Data $helper,
        \Magento\Sales\Api\Data\OrderInterface $order,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \Magento\Payment\Helper\Data $paymentHelper,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;
        $this->_order = $order;
        $this->date = $date;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_paymentHelper = $paymentHelper;
        $this->_addressRenderer = $addressRenderer;
        parent::__construct($context);
    }


    public function execute()
    {

      $params = $this->getRequest()->getParams();

        if (!$this->helper->getConfig('carriers/pickupinstore/active') || !isset($params["increment_id"]) || !$params["increment_id"] || !isset($params["hash"]) || !$params["hash"]) {
            return $this->_forward('noRoute');
        }

        $this->_order->loadByIncrementId($params["increment_id"]);

        if ($this->_order->getId() && $this->_order->getPickupInStoreHash() == $params["hash"] && $this->_order->getShippingMethod() == "pickupinstore_pickupinstore") {
          $this->_order->setPickupInStoreConfirmReceivedDate($this->date->gmtDate());
          $this->_order->save();
          $resultJson = $this->resultJsonFactory->create();
          $resultJson->setData(["message" =>__("You confirmed that the order ") . $params["increment_id"] . __(" was received.")]);
          $this->updateOrder();
            // Send mail to customer to inform that the package is available in store.
          $this->sendMail($this->_order);
          $this->_eventManager->dispatch('filoblu_storelocator_pickupinstore_confirm_received', ['order' => $this->_order]);

          return $resultJson;
        }
        else {
          return $this->_forward('noRoute');
        }
    }

    protected function sendMail($order) {
        if (!$this->helper->getConfig("carriers/pickupinstore/email_to_customer_enabled")) {
            return;
        }

        $this->_inlineTranslation->suspend();

        $billingAddress = $order->getBillingAddress();

        $templateVars = [
            'order' => $order,
            'billing' => $billingAddress,
            'payment_html' => $this->getPaymentHtml($order),
            'store' => $order->getStore(),
            'formattedShippingAddress' => $this->getFormattedShippingAddress($order),
            'formattedBillingAddress' => $this->getFormattedBillingAddress($order),
        ];

        $templateOptions = [
            'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
            'store' => $this->_storeManager->getStore()->getStoreId(),
        ];

        try {
            $transport = $this->_transportBuilder
                ->setTemplateIdentifier($this->helper->getConfig("carriers/pickupinstore/email_to_customer_template"))
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVars)
                ->setFrom($this->helper->getConfig("carriers/pickupinstore/sender_email_to_customer_identity"))
                ->addTo($order->getCustomerEmail())
                ->getTransport();
            $transport->sendMessage();

            $this->_inlineTranslation->resume();

        } catch (\Exception $e) {
            $this->_inlineTranslation->resume();
        }
    }

    protected function getPaymentHtml($order)
    {
        return $this->_paymentHelper->getInfoBlockHtml(
            $order->getPayment(),
            $this->_storeManager->getStore()->getId()
        );
    }

    protected function getFormattedShippingAddress($order)
    {
        return $order->getIsVirtual()
            ? null
            : $this->_addressRenderer->format($order->getShippingAddress(), 'html');
    }

    protected function getFormattedBillingAddress($order)
    {
        return $this->_addressRenderer->format($order->getBillingAddress(), 'html');
    }

    /**
     *  Update Order-status
     */
    public function updateOrder(){

        $order = $this->_order;
        $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING, true)->save();
        $order->setStatus("delivered_in_store")->save();
        $order->addStatusToHistory($order->getStatus(), 'Pick Up In Store - The order arrived in the store');
        $order->save();
    }
}
